c
c.....getopt - parses the command line for arguments, flags and options.
c
      logical function getopt (args, largs, nargs,
     &                         flgs, lflgs, nflgs,
     &                         opts, lopts, nopts, optarg)
c     The routine returns .false. if an error occurs, .true. otherwise.
c
c     Argument is any token in the commmand line that doesn't begin with
c     '-'. A standalone '-' will be considered a special argument, used
c     to skip an arg without filling it.
c
c     Flag is any char of a token that starts with a '-'.
c
c     Option is a flag that accepts an argument, which must be the token
c     following the option.
c
c     A '--' in the input line signals the end of options and flags (in
c     this way, arguments beginning with '-' are allowed).
c
c..Input parameters:
c
c  nargs ......... maximum number of allowed arguments.
c  nflgs ......... number of valid flags.
c  flgs .......... list of valid flags.
c  nopts ......... number of valid options.
c  opts .......... list of valid options.
c
c..Output parameters:
c
c  args() ........ returns the values of the specified arguments.
c  largs() ....... returns .true. on each specified argument (that is,
c                  '-' will increase the count of arguments but with
c                  a .false. in lflags).
c  lflgs() ....... returns a .true. in the position of the flags
c                  actually found in the command line.
c  lopts() ....... returns .true. in the position of used options.
c  optarg() ...... contains the arguments associated to the options
c                  actually found.
c
c..Example:
c
c  Given a program that expects for the next entrance:
c
c       myprog [input] [output] [-abcde] [-o arg1] [-s arg2]
c
c  where square braces [] contain optional arguments, the main program
c  may use the next call to getopt to parse the command line:
c
c       nargs = 2
c       nflgs = 5
c       flgs = 'abcde'
c       nopts = 2
c       opts = 'os'
c       if (.not. getopt (args, largs, nargs, flgs, lflgs, nflgs,
c      &            opts, lopts, nopts, optarg)) then
c          --- Something went wrong ---
c       endif
c
c PARAMETERS:
c
      integer           nargs, nflgs, nopts
      logical           largs(*), lflgs(*), lopts(*)
      character*(*)     args(*), optarg(*)
      character*(*)     flgs, opts
c
c LOCAL VARIABLES:
c
      integer           leng, iargc
      integer           iargs, iflgs, iopts
      integer           ntokens, itoken, ltoken, lp
      character*(128)   token
      logical           endopts
c
c.....get the number of tokens in the command line:
c
      iargs = 0
      ntokens = iargc ()
      if (ntokens.lt.0) then
        getopt = .false.
        return
      endif
c
c.....loop over tokens:
c
      endopts = .false.
      itoken = 1
      do while (itoken.le.ntokens)
        call getarg (itoken, token)
        itoken = itoken + 1
        ltoken = leng (token)
c
c.....is it an arg? (or we found the end of options)
c
        if (token(1:1).ne.'-' .or. endopts) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .true.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....dummy argument:
c
        else if (ltoken.eq.1) then
          if (iargs.lt.nargs) then
            iargs = iargs + 1
            args(iargs) = token(1:ltoken)
            largs(iargs) = .false.
          else
            write (0,*) 'getopt: too many arguments'
            getopt = .false.
            return
          endif
c
c.....end of options:
c
        else if (token(1:ltoken).eq.'--') then
          endopts = .true.
c
c.....everything else are flags and options:
c
        else
c
c.....loop for flags:
c
          do lp = 2, ltoken-1
            iflgs = 1
            do while (iflgs.lt.nflgs .and.
     &                token(lp:lp).ne.flgs(iflgs:iflgs))
              iflgs = iflgs + 1
            enddo
            if (iflgs.eq.nflgs .and.
     &          token(lp:lp).ne.flgs(iflgs:iflgs)) then
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
            lflgs(iflgs) = .true.
          enddo
c
c.....last one can be a flag or an option:
c
          lp = ltoken
          iflgs = 1
          do while (iflgs.lt.nflgs .and.
     &              token(lp:lp).ne.flgs(iflgs:iflgs))
            iflgs = iflgs + 1
          enddo
c
c.....it is a flag:
c
          if (iflgs.lt.nflgs .or.
     &        token(lp:lp).eq.flgs(iflgs:iflgs)) then
            lflgs(iflgs) = .true.
c
c.....not a flag:
c
          else
            iopts = 1
            do while (iopts.lt.nopts .and.
     &                token(lp:lp).ne.opts(iopts:iopts))
              iopts = iopts + 1
            enddo
c
c.....it is an option:
c
            if (iopts.ne.nopts .or.
     &          token(lp:lp).eq.opts(iopts:iopts)) then
              if (itoken.le.ntokens) then
                lopts(iopts) = .true.
                call getarg (itoken, token)
                itoken = itoken + 1
                ltoken = leng (token)
                optarg(iopts) = token(1:ltoken)
              else
                write (0,*)
     &             'getopt: missing argument in option -'//token(lp:lp)
                getopt = .false.
                return
              endif
c
c.....not an option:
c
            else
              write (0,*)
     &          'getopt: -'//token(lp:lp)//' is not a valid flag'
              getopt = .false.
              return
            endif
          endif
        endif
      enddo
c
c.....end of parsing ok:
c
      getopt = .true.
      return
      end function
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      integer function iargc ()
*c
*c.....iargc: substitute function for unix's library one. Returns the
*c     number of tokens in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered in the same token
*c
*      include           'error.inc'
*      character*(1)     blank,null,dquote
*      parameter ( blank=char(32) , null=char(0) , dquote=char(34) )
*      integer           leng, length, lp, argc
*      logical           done
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....get command line
*c
*mdc*endif
*mdc*if lahey
*      call getcl(line)
*      length=leng(line)
*mdc*elsif vms
*      include           '($clidef)'
*      call cli$get_value ('$line',line,length)
*mdc*endif
*mdc*if lahey vms
*c
*c.....skip initial spaces
*c
*      argc = 0
*      lp = 1
*      done = .false.
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).ne.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      if (.not.done) then
*        call error ('iargc', 'empty command line', warning)
*        iargc = -1
*        return
*      endif
*c
*c.....skip command name
*c
*      done = .false.
*      i = lp
*      do while (.not.done .and. lp.le.length)
*        if (line(lp:lp).eq.blank) then
*          done = .true.
*        else
*          lp = lp + 1
*        endif
*      enddo
*      token(argc) = line(i:lp-1)
*c
*c.....no arguments
*c
*      if (.not.done) then
*        iargc = 0
*        return
*      endif
*c
*c.....loop over tokens
*c
*      do while (lp.le.length)
*c
*c.....skip blanks
*c
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).ne.blank) then
*            done = .true.
*          else
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....get next token
*c
*        argc = argc + 1
*        if (argc.gt.margs)
*     &    call error ('iargc', 'too many tokens',faterr)
*        token(argc) = null
*        i = 1
*  10    continue
*        done = .false.
*        do while (.not.done .and. lp.le.length)
*          if (line(lp:lp).eq.blank .or. line(lp:lp).eq.dquote) then
*            done = .true.
*          else
*            token(argc)(i:i) = line(lp:lp)
*            i = i + 1
*            lp = lp + 1
*          endif
*        enddo
*c
*c.....end of command line
*c
*        if (.not.done) then
*          iargc = argc
*          return
*        endif
*c
*c.....token in double quotes; the goto takes into account abc" d "efg
*c     as one token
*c
*        if (line(lp:lp).eq.dquote) then
*          if (lp.eq.length) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          lp = lp + 1
*          done = .false.
*          do while (.not.done .and. lp.le.length)
*            if (line(lp:lp).eq.dquote) then
*              done = .true.
*            else
*              token(argc)(i:i) = line(lp:lp)
*              i = i + 1
*              lp = lp + 1
*            endif
*          enddo
*          lp = lp + 1
*          if (.not.done) then
*            write (0,*) 'iargc: unmatched '//dquote
*            iargc = -1
*            return
*          endif
*          goto 10
*        endif
*      enddo
*c
*c.....return the number of arguments
*c
*      iargc = argc
*      return
*      end
*mdc*endif
c
c ----------------------------------------------------------------------
c
*mdc*if lahey vms
*      subroutine getarg (itoken, rtoken)
*c
*c.....getarg: substitute function for unix's library one. Returns the
*c     token number itoken in the command line; tokens are separated by
*c     blanks; in addition, everything between double quotes is
*c     considered to be in the same token, after removing the quotes
*c
*      integer           itoken
*      character*(*)     rtoken
*c
*      integer           length, argc
*      character*(1)     null
*      parameter ( null = char(0) )
*      parameter ( margs = 16 )
*      character*(128)   line, token(0:margs)
*      common /commline/ length, argc, line, token
*c
*c.....test for valid itoken
*c
*      if (itoken.lt.0 .or. itoken.gt.argc) then
*        write (0,*) 'getarg: invalid token number ',itoken
*        rtoken = null
*      else
*        rtoken = token(itoken)
*      endif
*      return
*      end
*mdc*endif
c---------------------------------------------------------------------
c
c.....leng - obtains the length of the string, assuming that the blanks
c     at the end are dummy.
c
      integer function leng (string)
c
      character*(*)  string
      character*(1)  blank, null
      parameter      (blank = ' ')
      null = char(0)
      do 10 i=len(string),1,-1
         if (string(i:i).ne.blank .and. string(i:i).ne.null) then
            leng=i
            return
         endif
 10   continue
      leng=0
      return
      end function
c---------------------------------------------------------------------
      subroutine error (routine,message,errortype)
c
      use datos, only : faterr, warning
c
      character*(*)       routine,message
      integer             errortype
      character*(20)      chtype
c
      if (errortype.eq.faterr) then
         chtype='fatal error'
      else if (errortype.eq.warning) then
         chtype='warning'
      else
         chtype='unknown error'
      endif
      write (*,100) chtype(1:leng(chtype)),routine,message
      if (errortype.eq.faterr) stop
c
 100  format (/,1x,'*** '/,1x,'*** ',a,' at ',a,' :'/,
     &          1x,'*** ',a/,1x,'***')
c
      end subroutine
c---------------------------------------------------------------------
      subroutine getlin(c1,inpu,outu)
c     This subroutine gets a  non-blank line of command in c1, if the 
c     line terminates with a "-" then the next line is appended to it 
c     and so on, until the maximum number of lines "nline" of len1gth 
c     "len1" is read.
      character*(*) c1
      character*350 line
      logical check,trim
      external check,trim
      character*80 strg(10)
      integer inpu,outu
      integer begin,endlin
      logical done

      c1=' '
      line=' '
      begin=1
      lenc1=len(c1)
      endlin=0
      done=.false.

 1000 read(inpu,100,end=2000) line
  100 format(a)

      call clean(line)
      if(trim(line))then

      call fend(line,endlin)
      if(check(line(endlin:),'-'))then
      done=.false.
      else
      done=.true.
      endif

      c1(begin:)=line(1:endlin)
      begin=begin+endlin
      if(begin.gt.lenc1)then
      write(abs(outu),'(a)')
     ,' * GETLIN * error, command line truncated'
      done=.true.
      endif

      else
      done=.false.
      endif

      if(done)then
      call subst(c1,strg,outu)
      return !that was the last line
      else
      goto 1000 !append a new line
      endif


c     END-OF-FILE encountered
 2000 c1='*END*'
      write(abs(outu),'(a)')
     ,' * GETLIN * error, end of file encountered'
      stop
      return

      end subroutine
c---------------------------------------------------------------------
      subroutine nxtchn(c1,c2,c3)
c     Look in the chain c1 for the next chain coming  before the 
c     chain c2 (possibly a blank) and put it in the chain c3.  
c     The chain c2 and c3 are then then removed from c1.
      character*(*) c1,c2,c3
      logical check,trim
      external check,trim

      if(trim(c1))then
      c3=' '
      i2=index(c1,c2)
      if(i2.eq.0) return ! c2 is not in c1
      c3=c1(1:i2-1) !save the word c3
      c1=c1(i2+len(c2):len(c1)) !remove it from the chain c1
      endif
      return

      end subroutine
c---------------------------------------------------------------------
      subroutine upper(c1)
c     UPPER CASE CONVERSION
c     The fortran function ICHAR(a) returns the asci number corresponding to 
c     the character (a).  The function CHAR(i) returns the character 
c     corresponding to the asci number i.
c     For alphabetic characters:
c     A = 65, B = 66, ..., Z = 90
c     a = 97, b = 98, ..., z = 122
      character*(*) c1
      integer lenc1,ic1

      lenc1=len(c1)
      do 1 i=1,lenc1
      ic1=ichar(c1(i:i))
      if((ic1.ge.97).and.(ic1.le.122))then
      c1(i:i)=char(ic1-32)
      endif
    1 continue
      return

      end subroutine
c---------------------------------------------------------------------
      subroutine gtipar(c1,c2,ipar,idef)
c     This subroutine get the integer parameter "ipar"
c     coming just after the word c2 in the chain c1.
c     At the end the word and the parameter are removed
c     from the c1.  If nothing was there, the default "idef"
c     is used.
      character*(*) c1,c2
      character*25 junk
      integer ipar
      logical trim
      external trim
      external icnvrt
      logical error
      call getwrd(c1,c2,junk)
      if(trim(junk))then
      ipar=icnvrt(junk,error)
      if(error) ipar=idef
      else
      ipar=idef
      endif
      return

      end subroutine
c---------------------------------------------------------------------
      subroutine getwrd(c1,c2,c3)
c     Look in the chain c1 for the word coming 
c     just after c2 and put it in the c3.  The word
c     is then removed form c1.  This subroutine is used
c     to get the name of files to open.
      character*(*) c1,c2,c3
      integer lenc1,lenc2,i1,i2
      c3=' '
      lenc1=len(c1)
      lenc2=len(c2)
      i1=index(c1,c2)
      if(i1.gt.0)then
      i2=index(c1(i1+lenc2:lenc1),' ')+i1+lenc2-1
      c1(i1:lenc1)=c1(i2:lenc1) !remove the word until the next ' '
      call nxtchn(c1(i1:lenc1),' ',c3)
      endif
      return

      end subroutine
c--------------------------------------------------------------------
      subroutine clean(c1)
c     This subroutine removes any comments placed after !
      character*(*) c1
      length=len(c1)
      i1=index(c1,'!')
      if(i1.ne.0) then
      c1(1:length)=c1(1:i1-1)
      endif
      return

      end subroutine
c--------------------------------------------------------------------
      subroutine fend(c1,i1)
c     This subroutine finds the last non-blank character in a chain
      character*(*) c1
      lenc1=len(c1)
      do 1 i=lenc1-1,1,-1
      if(c1(i:i+1).ne.' ')then
      i1=i
      return
      endif
    1 continue
      i1=1
      return

      end subroutine
c--------------------------------------------------------------------
      logical function check(c1,c2)
c     This subroutine verifies if c2 is in c1 and remove it
      character*(*) c1,c2
      external trim
      logical junk,trim
      integer i1,i2,lenc1,lenc2
      i1=index(c1,c2)
      check=i1.ne.0
      if(check)then
      lenc1=len(c1)
      lenc2=len(c2)
      i2=index(c1(i1+lenc2:lenc1),' ')+i1+lenc2-1
      c1(i1:lenc1)=c1(i2:lenc1) !remove the word until the next ' '
      junk=trim(c1)
      endif
      return

      end function
c--------------------------------------------------------------------
      subroutine subst(com,strg,outun)
c     This subroutine takes care of the substitutions of internal parameters
      character*(*) com
      character*80 strg(10),wrd80
      logical error
      integer outun

c     Make the substitutions first
      istrg = 0
 1000 i1 = index(com,'@')
      if (i1.ne.0) then
        istrg = icnvrt(com(i1+1:i1+1),error)
        if (.not.error) then
          call fend(strg(istrg),lstrg)
          lencom = len(com)
          wrd80 = strg(istrg)
          com(i1:lencom) = wrd80(1:lstrg)//com(i1+2:lencom)
          if (outun.gt.0) write(outun,100) istrg,wrd80(1:lstrg)
  100     format(' Parameter #',i2,' substituted by "',a,'"')
          goto 1000  !make shure all parameters have been changed
        endif
      endif
      if (outun*istrg.gt.0) write(outun,*)
      return

      end subroutine
c--------------------------------------------------------------------
      logical function trim(c1)
c     This subroutine removes the blanks on the left
      character*(*) c1
      length=len(c1)
      do 1 i=1,length
      trim=c1(i:i).ne.' '
      if(trim)then
      c1(1:length)=c1(i:length)
      return
      endif
    1 continue
      return

      end function
c--------------------------------------------------------------------
      function icnvrt(c1,error)
      integer icnvrt
      character*(*) c1
      character fmt*5
      integer lenc1
      logical error
c
      error=.false.
      call fend(c1,lenc1)
      if(lenc1.eq.0)then
       write(*,*) '*ICNVRT* zero length string'
       icnvrt=0
       return
      endif
      write(fmt,100) '(i',lenc1,')'
  100 format(a,i2,a)
      read(c1(1:lenc1),fmt,err=1000) icnvrt
      return
 1000 error=.true.
      write(*,*)'*ERROR IN CONVERSION*  c1=',c1(1:lenc1)
      icnvrt=0
      return

      end function
